<?php
/* Copyright 2007-2009 Xavier 'Xr' Dalem
 * 
 * This file is part of Demyo. Demyo is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License version 3,
 * as published by the Free Software Foundation.
 * 
 * Demyo is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU GPLv3 for more details.
 */
class AlbumsController extends AppController {
	public $scaffold;
	public $helpers = array('Form', 'List', 'Menu', 'MyPaginator', 'Quicksearch', 'Shadowbox', 'Text');
	public $components = array('PhpThumb');
	public $paginate = array('limit' => 42, 'page' => 1, 'order'=>array('Series.name' => 'asc', 'Album.cycle' => 'asc', 'Album.number' => 'asc'), 'recursive' => 1);
	public $uses = array('Album', 'Author', 'Binding', 'Collection', 'Derivative', 'Image', 'Publisher', 'Series', 'Tag'); 

	function index() {
		$this->paginate['limit'] = ($perColumn = Configure::read('Demyo.itemsPerIndexColumns')) * Configure::read('Demyo.columnsPerIndexPage');
		$this->set('perColumn', $perColumn);
		
		$filters = $this->getFilters(array('withBinding', 'fromPublisher', 'fromCollection'));
		extract($filters);
		
		$scope = array();
		$relationsToKeep = array('belongsTo' => array('Series'));
		if ($withBinding !== null) {
			array_push($scope, array('Binding.id' => $withBinding));
			$relationsToKeep['belongsTo'][] = 'Binding';
			$this->pageTitle = __s('Viewing albums of binding \'%s\'', $this->Binding->getName($withBinding));
		}
		if ($fromPublisher !== null) {
			array_push($scope, array('Publisher.id' => $fromPublisher));
			$relationsToKeep['belongsTo'][] = 'Publisher';
			$this->pageTitle = __s('Viewing albums from %s', $this->Publisher->getName($fromPublisher));
		}
		if ($fromCollection !== null) {
			array_push($scope, array('Collection.id' => $fromCollection));
			$relationsToKeep['belongsTo'][] = 'Collection';
			$this->pageTitle = __s('Viewing albums from %s', $this->Collection->getName($fromCollection));
		}
		
		if (isset($this->params['named']['startsWith'])) {
			$this->paginate['page'] = $this->Album->findPageNumberStartsWith('Series.name', $this->params['named']['startsWith'], $this->paginate['limit'], $scope);
		}
		
		$this->params['pass'] = array_filter($filters);
		$this->Album->unbindAll($relationsToKeep, false);
		$this->set('albums', $this->paginate('Album', $scope));
	}

	function view($id = null) {
		if ($id == null)
			$this->redirect('/albums');

		$this->Album->id = $id;
		$this->Album->contain(array(
			'Publisher.id', 'Publisher.name',
			'Collection.id', 'Collection.name',
			'Binding',
			'Series.id', 'Series.name',
			'Artist.id', 'Artist.name', 'Artist.fname', 'Artist.nickname',
			'Writer.id', 'Writer.name', 'Writer.fname', 'Writer.nickname',
			'Colorist.id', 'Colorist.name', 'Colorist.fname', 'Colorist.nickname',
			'Tag',
			'Cover',
			'Image',
			'Borrower',
			'Derivative.id',
			'Derivative.Image'
		));
		if (! $album = $this->Album->read())
			$this->errorNoSuchEntry(__('This album does not exist.', true));
		
		// Fake album data as if the images from the derivatives were actually
		// from the album
		foreach ($album['Derivative'] as &$deriv)
			$album['Image'] = array_merge($album['Image'], $deriv['Image']);
		unset($album['Derivative']);
		
		// Generate thumbnails
		if (! empty($album['Cover']['id']))
			$album['Cover']['thumb_url'] = $this->PhpThumb->getThumbnail(IMAGES.$album['Cover']['url']);
		foreach($album['Image'] as &$image)
			$image['thumb_url'] = $this->PhpThumb->getThumbnail(IMAGES.$image['url']);
		
		$album['Album']['derivative_count'] = $this->Derivative->getDerivativesCount('Album', $id);
		
		$this->set('album', $album);
	}

	function add() {
		if (!empty($this->data)) {
			$this->data['Tag']['Tag'] = $this->Tag->getFromCSV($this->data['Tag']['Tag']);
			if ($this->Album->save($this->data)) {
				$this->smartFlash(__('The album has been added', true),
					'/albums/view/'.$this->Album->id);
			}
		}
		
		else {
			// Hack POST array to select default values, some of them being
			// guessed from the most recent album (they rarely change)
			
			if (isset($this->params['named']['series'])) {
				// Series			
				$this->data['Album']['series_id'] = $this->params['named']['series'];

				$this->Album->contain(array(
					'Writer.id', 'Artist.id', 'Colorist.id'
				));
				$lastAlbum = $this->Album->find('first', array(
					'conditions' => array('series_id' =>  $this->params['named']['series']),
					'order' => array('cycle' => 'DESC', 'number' => 'DESC'),
				));
				
				if (! empty($lastAlbum)) {
					// Publisher
					$this->data['Album']['publisher_id'] = $lastAlbum['Album']['publisher_id'];
					
					// Collection
					$this->data['Album']['collection_id'] = $lastAlbum['Album']['collection_id'];
					
					// Binding
					$this->data['Album']['binding_id'] = $lastAlbum['Album']['binding_id'];
					
					// Dimensions
					if (! empty($lastAlbum['Album']['width']))
						$this->data['Album']['width'] = $lastAlbum['Album']['width'];
					if (! empty($lastAlbum['Album']['height']))
						$this->data['Album']['height'] = $lastAlbum['Album']['height'];
					
					// Authors
					$this->data['Colorist'] = $lastAlbum['Colorist'];
					$this->data['Artist'] = $lastAlbum['Artist'];
					$this->data['Writer'] = $lastAlbum['Writer'];
				}
			}
			
			
			$this->generateLists();
		}
	}
	
	function edit($id = null) {
		if (empty($this->data)) {
			$this->Album->id = $id;
			if (! $this->data = $this->Album->read())
				$this->errorNoSuchEntry(__('This album does not exist.', true));
			
			// Fix dates
			foreach (array('first_edition', 'this_edition', 'purchase_date') as $date)
				if (isset($this->data['Album'][$date]) && $this->data['Album'][$date] === '0000-00-00')
					$this->data['Album'][$date] = '';

			// Fix tags: fetch names, format as CSV, remove old info. Name-to-id conversion is done when saving
			$tags = array();
			foreach($this->data['Tag'] as $tag)
				array_push($tags, $tag['name']);
			unset($this->data['Tag']);
			$this->data['Tag']['Tag'] = implode(',', $tags);
			
			$this->generateLists();
		}
		
		else {
			// Recover tag IDs, as we previously gave names
			$this->data['Tag']['Tag'] = $this->Tag->getFromCSV($this->data['Tag']['Tag']);
			if ($this->Album->save($this->data)) {
				$this->smartFlash(__('The album has been updated', true),
					'/albums/view/'.$this->data['Album']['id']);
			}
		}
	}
	
	private function generateLists() {
		$this->set('seriesList', $this->Series->find('list'));
		$this->set('publishersList', $this->Publisher->find('list'));
		$this->set('collectionsList', $this->Collection->find('list'));
		$this->set('bindingsList', $this->Binding->find('list'));
		$this->set('authorsList', $this->Author->find('list'));
		$this->set('tagsList', $this->Tag->find('list'));
		$this->set('imagesList', $this->Image->find('list'));
	}
	
	function fromSeries($id = null) {
		$this->layout = 'ajax';
		
		if ($id != null)
			$this->set('albums', $this->Album->find('list', array('conditions' => 'Album.series_id='.$id, 'order' => 'cycle ASC, number ASC')));
		else
			$this->set('albums', array());
	}
}

?>
