<?php
/* Copyright 2007-2009 Xavier 'Xr' Dalem
 * 
 * This file is part of Demyo. Demyo is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License version 3,
 * as published by the Free Software Foundation.
 * 
 * Demyo is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU GPLv3 for more details.
 */
/** @todo Handle errors in a better way than just flashing messages */
class BorrowersController extends AppController {
	public $scaffold;
	public $uses = array('Borrower', 'Album', 'AlbumsBorrower');
	public $helpers = array('Menu', 'MyPaginator');
	
	function index() {
		$showAll = !isset($this->params['named']['onlyCurrent'])
			|| $this->params['named']['onlyCurrent'] === 'false';
		
		$fromBorrower = false;
		if (isset($this->params['named']['borrower']))
			$fromBorrower = $this->params['named']['borrower'];
			
		// Prepare paginator
		$page = isset($this->params['named']['page'])?$this->params['named']['page']:1;
		$total = $this->AlbumsBorrower->getLoansCount($fromBorrower, $showAll);
		$limit = 20;
		
		$this->prepareManualPagination('Borrower', $page, $limit, $total,
			array('onlyCurrent' => $showAll?'false':'true', 'borrower' => $fromBorrower));
			
		
		$this->set('loans', $this->AlbumsBorrower->getLoans($fromBorrower, $showAll, $page, $limit));
		$this->set('showingAll', $showAll);
		$this->set('fromBorrower', $fromBorrower);
	}
	
	function view($id = null) {
		$this->Borrower->id = $id;
		$this->Borrower->recursive = -1;
		if (! $borrower = $this->Borrower->read())
			$this->errorNoSuchEntry(__('This borrower does not exist.', true));
		$this->set('borrower', current($borrower));
		$this->set('hasAlbums', $this->AlbumsBorrower->hasAlbums($id));
    }
    
	function edit($id = null) {
		if (empty($this->data)) {
			$this->Borrower->id = $id;
			$this->Borrower->recursive = -1;
			if (! $this->data = $this->Borrower->read())
				$this->errorNoSuchEntry(__('This borrower does not exist.', true));
		}
		
		else {
			if ($this->Borrower->save($this->data)) {
				$this->smartFlash(__('The borrower has been updated', true),
					'/borrowers/view/'.$this->data['Borrower']['id']);
			}
		}
	}
	
	public function lend() {
		// Where to go back in case of error
		$backUrl = empty($_SERVER['HTTP_REFERER'])?'/albums':$_SERVER['HTTP_REFERER'];
		
		if (empty($this->data))
			return $this->flash(__('No album selected for loan', true), $backUrl);
		
		// Look for borrower
		if (empty($this->data['Borrower']['id'])) {
			if (empty($this->data['Borrower']['name']))
				return $this->flash(__('No borrower specified', true), $backUrl);
			$borrowerID = $this->Borrower->getIDOrAdd($this->data['Borrower']['name']);
		} else
			$borrowerID = $this->data['Borrower']['id'];
		
		// Check date
		$date = empty($this->data['date'])?date('Y-m-d'):$this->data['date'];
		if (!Validation::date($date))
			return $this->flash(__s('Invalid date: %s', $date), $backUrl);
		
		// Check which albums were selected
		$albums = array();
		foreach ($this->data['Album'] as $albumID => $state) {
			if ($state === 'on') {
				array_push($albums, $albumID);
			}
		}
		
		try {
			$this->AlbumsBorrower->lend($borrowerID, $albums);
		} catch (Exception $e) {
			return $this->flash($e->getMessage(), $backUrl);
		}
		
		$this->flash(__('The albums have been marked as lended', true), $backUrl);
	}
	
	public function getBack($albumId = null) {
		// Where to go back in case of error
		$backUrl = empty($_SERVER['HTTP_REFERER'])?'/albums':$_SERVER['HTTP_REFERER'];
		
		// Recover album IDs...
		$albums = array();
		
		// ... from GET
		if ($albumId !== null)
			$albums[] = $albumId;
			
		// ... or from POST
		else if (! empty($this->data)) {
			foreach ($this->data['Album'] as $albumID => $state)
				if ($state === 'on')
					array_push($albums, $albumID);
		}
		
		// ... if nothing was found, show currently lended albums
		else {
			$this->redirect('/borrowers/index/onlyCurrent:/', null, true);
			return;
		}
				
		// Check date, or generate it
		$date = empty($this->data['date'])?date('Y-m-d'):$this->data['date'];
		if (!Validation::date($date))
			return $this->flash(__s('Invalid date: %s', $date), $backUrl);
		
		try {
			$this->AlbumsBorrower->getBack($albums, $date);
		} catch (Exception $e) {
			return $this->flash($e->getMessage(), $backUrl);
		}
		
		$this->flash(__('The albums have been marked as returned', true), $backUrl);
	}
}

?>
