<?php
/* Copyright 2007-2009 Xavier 'Xr' Dalem
 * 
 * This file is part of Demyo. Demyo is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License version 3,
 * as published by the Free Software Foundation.
 * 
 * Demyo is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU GPLv3 for more details.
 */
class DerivativesController extends AppController {
	public $scaffold;
	public $uses = array('Derivative', 'Album', 'Author', 'DerivativeType', 'Image', 'Series', 'Source');
	public $components = array('PhpThumb', 'RequestHandler');
	public $helpers = array('Form', 'jQuery', 'Menu', 'MyPaginator', 'Quicksearch', 'Shadowbox', 'Text');
	public $paginate = array('limit' => 45,
		'page' => 1,
		'order'=>array('Series.name' => 'ASC', 'Album.cycle' => 'ASC', 'Album.number' => 'ASC', 'Source.name' => 'ASC'),
		'fields' => array(
			'Derivative.id', 'Derivative.album_id', 'Derivative.colors', 'Derivative.source_id', 'Derivative.number', 'Derivative.total',
			'Series.id', 'Series.name',
			'Album.title', 'Album.first_edition',
			'DerivativeType.*',
			'Source.id', 'Source.name',
			'Artist.id', 'Artist.name', 'Artist.fname', 'Artist.nickname'
		),
		'recursive' => 1);
	
	public function index() {
		$this->paginate['limit'] = Configure::read('Demyo.Gallery_thumbsPerPage');
		
		$isAjax = $this->RequestHandler->isAjax();
		$this->set('isAjax', $isAjax); 
		if ($isAjax) {
			$this->layout = 'ajax';
		}
		
		$filters = $this->getFilters(array('fromSeries', 'fromAlbum', 'fromArtist', 'fromType', 'fromSource'));
		extract($filters);
		
		$scope = array();
		if ($fromSeries !== null) {
			array_push($scope, array('Series.id' => $fromSeries));
			$this->pageTitle = __s('Viewing derivatives from %s', $this->Series->getName($fromSeries));
		}
		if ($fromAlbum !== null) {
			array_push($scope, array('Album.id' => $fromAlbum));
			$this->pageTitle = __s('Viewing derivatives from %s', $this->Album->getName($fromAlbum));
		}
		if ($fromArtist !== null) {
			array_push($scope, array('Artist.id' => $fromArtist));
			$this->pageTitle = __s('Viewing derivatives from %s', $this->Author->getName($fromArtist));
		}
		if ($fromType !== null) {
			array_push($scope, array('DerivativeType.id' => $fromType));
			$this->pageTitle = __s('Viewing derivatives of type \'%s\'', $this->DerivativeType->getName($fromType));
		}
		if ($fromSource !== null) {
			array_push($scope, array('Source.id' => $fromSource));
			$this->pageTitle = __s('Viewing derivatives from \'%s\'', $this->Source->getName($fromSource));
		}
		
		if (empty($this->pageTitle) || $isAjax)
			$this->pageTitle = __('Derivatives', true);
		
		if (isset($this->params['named']['startsWith'])) {
			$this->paginate['page'] = $this->Derivative->findPageNumberStartsWith('Series.name', $this->params['named']['startsWith'], $this->paginate['limit'], $scope);
		}
		
		$derivs =  $this->paginate('Derivative', $scope);
		foreach ($derivs as &$deriv)
			foreach ($deriv['Image'] as &$image)
				$image['thumb_url'] = $this->PhpThumb->getThumbnail(IMAGES.$image['url']);
		
		$this->params['pass'] = array_filter($filters);
		$this->set('derivatives', $derivs);
	}
	
	function view($id = null) {
		if ($id === null)
			$this->redirect('/derivatives');

		$this->Derivative->id = $id;
		if (! $derivative = $this->Derivative->read())
			$this->errorNoSuchEntry(__('This derivative does not exist.', true));
		
		foreach($derivative['Image'] as &$image)
				$image['thumb_url'] = $this->PhpThumb->getThumbnail(IMAGES.$image['url']);
		
		$this->set('derivative', $derivative);
	}
	
	function add() {
		if (!empty($this->data)) {
			$this->data['Derivative']['source_id'] = $this->Source->getOrAdd($this->data['Source']['name']);
			unset($this->data['Source']);
			
			if ($this->Derivative->save($this->data)) {
				$this->smartFlash(__('The derivative has been added', true),
					'/derivatives/view/'.$this->Derivative->id);
			}
		}
		
		else {
			$this->generateLists();
			
			// Hack post array to auto-select some fields
			if (isset($this->params['named']['toAlbum'])) {
				$album = $this->params['named']['toAlbum'];
				$this->Album->id = $album;
				$this->Album->unbindAll(array('belongsTo' => array('Series'), 'hasAndBelongsToMany' => array('Artist')), true);
				$data = $this->Album->read();
				$this->data['Derivative']['series_id'] = $data['Series']['id'];
				$this->data['Derivative']['album_id'] = $album;
				$this->data['Derivative']['purchase_date'] = ($data['Album']['purchase_date'] === '0000-00-00')?'':$data['Album']['purchase_date'];
				$this->data['Derivative']['artist_id'] = empty($data['Artist'])?0:$data['Artist'][0]['id'];
			}
			else if (isset($this->params['named']['toSeries']))
				$this->data['Derivative']['series_id'] = $this->params['named']['toSeries'];
		}
	}
	
	function edit($id = null) {
		if (empty($this->data)) {
			$this->Derivative->id = $id;
			if (! $this->data = $this->Derivative->read())
				$this->errorNoSuchEntry(__('This derivative does not exist.', true));
			
			// Fix date, see #34
			if (isset($this->data['Derivative']['purchase_date']) && $this->data['Derivative']['purchase_date'] === '0000-00-00')
				$this->data['Derivative']['purchase_date'] = '';
				
			$this->generateLists($this->data['Series']['id']);
		}
		
		else {
			// Don't trust given series_id; fetch it from the album
			if (! empty($this->data['Derivative']['album_id'])) 
				$this->data['Derivative']['series_id'] = current(current(
					$this->Album->find('first', array('fields' => 'series_id',
						'conditions' => array('id' => $this->data['Derivative']['album_id']),
						'recursive' => -1))));
			
			$this->data['Derivative']['source_id'] = $this->Source->getOrAdd($this->data['Source']['name']);
			unset($this->data['Source']);
			if ($this->Derivative->save($this->data)) {
				$this->smartFlash(__('The derivative has been updated', true),
					'/derivatives/view/'.$this->data['Derivative']['id']);
			}
		}
	}
	
	private function generateLists($seriesID = -1) {
		$seriesList = $this->Series->find('list');
		
		if ($seriesID === -1)
			$seriesID = current(array_keys($seriesList));
		
		$this->set('seriesList', &$seriesList);
		$this->set('authorsList', $this->Author->find('list'));
		$this->set('typesList', $this->DerivativeType->find('list'));
		$this->set('imagesList', $this->Image->find('list'));
		$this->set('albumsList', $this->Album->find('list', array('conditions' => 'Album.series_id='.$seriesID)));
	}
	
	public function labels() {
		$this->layout = 'printable';
		
		$this->Derivative->unbindModel(array('hasAndBelongsToMany' => array('Image')));
		$this->set('derivatives', $this->Derivative->find('all', array('fields' => array(
				'Series.name',
				'Album.cycle', 'Album.number', 'Album.title', 'Album.first_edition',
				'Artist.name', 'Artist.fname', 'Artist.nickname',
				'Source.name',
				'DerivativeType.name',
				'Derivative.number', 'Derivative.total', 'Derivative.authors_copy', 'Derivative.restricted_sale'),
			 'recursive' => 1, 'order' => $this->paginate['order'])));
	}
}

?>
