<?php
/* Copyright 2007-2009 Xavier 'Xr' Dalem
 * 
 * This file is part of Demyo. Demyo is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License version 3,
 * as published by the Free Software Foundation.
 * 
 * Demyo is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU GPLv3 for more details.
 */
class Album extends AppModel {
	public $name = 'Album';
	public $actsAs = array('Containable', 'SimpleXML' => array('useHABTM' => true));
	public $hasMany = array('Derivative');
	public $belongsTo = array('Publisher', 'Collection', 'Binding', 'Series',
		/* An album doesn't actually belong to a cover, but we want the ID to
		 * be part of Album and not of Cover (which doesn't exist and is an alias
		 * to Image)
		 */
		'Cover' => array('className' => 'Image', 'dependent' => false));
	public $hasAndBelongsToMany = array(
		'Artist' =>
			array('className' => 'Artist',
				'joinTable' => 'albums_artists',
				'foreignkey' => 'album_id',
				'associationForeignKey' => 'artist_id',
				'order'=>'name ASC',
				'uniq' => true,
			),
		'Writer' =>
			array('className' => 'Writer',
				'joinTable' => 'albums_writers',
				'foreignkey' => 'album_id',
				'associationForeignKey' => 'writer_id',
				'order'=>'name ASC',
				'uniq' => true,
			),
		'Colorist' =>
			array('className' => 'Colorist',
				'joinTable' => 'albums_colorists',
				'foreignkey' => 'album_id',
				'associationForeignKey' => 'colorist_id',
				'order'=>'name ASC',
				'uniq' => true,
			),
		'Tag' =>
			array('className' => 'Tag',
				'order' => 'Tag.name ASC',
				'uniq' => true,
			),
		'Image' => array('className' => 'Image'),
		'Borrower' => array(
			'className' => 'Borrower',
			'condition' => '(AlbumsBorrower.return_date IS NULL OR AlbumsBorrower.return_date=\'0000-00-00\')',
			'with' => 'AlbumsBorrower'
		)
	);
	
	/* After find callback, builds a meta_title based on the Album title, number
	 * and Series.
	 *
	 * @param mixed $results The results of the find operation
	 * @param boolean $primary Whether this model is being queried directly (vs. being queried as an association)
	 * @return mixed Result of the find operation
	 * @access public
	 */
	/* Left just in case, but of no use: the different uses we have of a meta
	 * are too different: we don't always need both the series and cycle+number
	 * and the title already conveys enough information
	public function afterFind($results, $primary = false) {
		foreach ($results as &$result) {
			if (isset($result[$this->name])) { // result was ['Model' => ...]
				$data =& $result[$this->name];
				if (isset($data['Series']) && isset($data['Series']['name']))
					$series = $data['Series']['name'];
				else if (isset($result['Series']) && isset($result['Series']['name']))
					$series = $result['Series']['name'];
			}
			else if (isset($result['title'])) // result was ['field1', 'field2']
				$data =& $result;
			else
				continue;

			if (empty($data)) // no matching record
				continue;
			
			if (! is_array(current($data))) { // result was ['Model' => ['field1', 'field2']]
				$title = '';
				if (isset($series))
					$title = $series.' - ';
				if (! empty($data['cycle']))
					$title .= $data['cycle'].'.';
				if (! empty($data['number']))
					$title .= $data['number'].' - ';
				if (! empty($data['title']))
					$title .= $data['title'];
					
				if (! empty($title))
					$data['meta_title'] = $title;
			}
			// else result was ['Model' => [0 => ['field1', 'field2']]], already
			// handled in a previous method call (the call for the sub-items)
		}
		
		return $results;
	}*/
	
	public function getImages($id) {
		$this->unbindAll(array('belongsTo' => array('Cover'), 'hasAndBelongsToMany' => array('Image')));
		$album = $this->read(null, $id);
		return array_merge(array($album['Cover']), $album['Image']);
	}
	
	public function getWorth($conditions = null) {
		$sum = $this->find($conditions, "SUM(price)", null, -1);
		return (float)$sum[0]['SUM(price)'];
	}
	
	public function getAlbumCount($model, $id) {
		if (! isset($this->belongsTo[$model])) {
			$this->log("$model is not a associated to {$this->name}", LOG_WARNING);
			return 0;
		}
		
		$model = Inflector::underscore($model);
		return $this->find('count', array(
			'conditions' => array("{$this->name}.{$model}_id" => $id),
			'recursive' => -1
		));
	}
	
	public static function sortWithSeries($a, $b) {
		if ($a['Series']['name'] != $b['Series']['name'])
			return ($a['Series']['name'] > $b['Series']['name']) ? +1 : -1;

		if ($a['cycle'] != $b['cycle'])
			return ($a['cycle'] > $b['cycle']) ? +1 : -1;
		
		if ($a['number'] != $b['number'])
			return ($a['number'] > $b['number']) ? +1 : -1;
		
		return 0;
	}
}

?>
