<?php
/* Copyright 2007-2009 Xavier 'Xr' Dalem
 * 
 * This file is part of Demyo. Demyo is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License version 3,
 * as published by the Free Software Foundation.
 * 
 * Demyo is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU GPLv3 for more details.
 */

class HelpBrowserHelper extends AppHelper {
	public $helpers = array('jQuery');
	private $helpIndexes = array();

	public function directLink($id, $name = false) {
		if ($id === 'index') {
			$index = $this->getHelpIndex();
			$url = Router::url(array('controller' => 'help', 'action' => 'index'));
			if ($name === false) $name = $index['index-name'];
			return "<a href=\"$url\" title=\"$name\" class=\"help-link\">$name</a>";
		}
		
		$entry = $this->getEntry($id);
		
		if ($entry === false)
			return false;
		
		$seeDoc = __('See documentation on this topic', true);
		if ($name === false) $name = $entry['name'];
		
		$url = $this->getURL($id, $entry);
		
		return "<a href=\"$url\" title=\"$seeDoc\" class=\"help-link\">$name</a>";
	}

	public function pageLink($id) {
		$entry = $this->getEntry($id);
		
		if ($entry === false)
			return false;
		
		$seeDoc = __s('See documentation about \'%s\'', $entry['name']);
		
		$url = $this->getURL($id, $entry);
		
		return "<a href=\"$url\" title=\"$seeDoc\" class=\"help-page-link\" ".
			"onclick=\"window.open('$url', '', 'height=500,width=650,top=210,left=40,location=no,menubar=no,resizable=yes,scrollbars=yes,toolbar=no,status=no,directories=no'); return false;\">?</a>";
	}

	private function getURL($id, $entry) {
		if(!empty($entry['local-id']))
			$id = $entry['local-id'];
		if(!empty($entry['anchor']))
			$id .= '#'.$entry['anchor'];
		return Router::url(array('controller' => 'help', 'action' => 'view', $id));
	}

	/** Gets an entry information, or the fallback information if none is
	 * available.
	 */
	private function getEntry($id, $lang = false) {
		$index = $this->getHelpIndex($lang);
		
		if (isset($index['ids'][$id]))
			return $index['ids'][$id];
		
		else if (isset($index['fallback'])) { // Fallback
			$index = $this->getHelpIndex($index['fallback']);
			if (isset($index['ids'][$id]))
				return $index['ids'][$id];
			else
				return false;
		}
		
		else
			return false;
	}

	private function getHelpIndex($lang = false) {
		if ($lang === false)
			$lang = Configure::read('Demyo.lang');
		
		// Load from memory
		if (isset($this->helpIndexes[$lang]))
			return $this->helpIndexes[$lang];
		
		// Load from disk cache
		$cacheFile = CACHE."help-$lang.php";
		if (file_exists($cacheFile)) {
			include($cacheFile);
			$this->helpIndexes[$lang] = $helpIndex;
			return $helpIndex;
		}
		
		return false;
	}
}
?>
