<?php
/* Copyright 2007-2009 Xavier 'Xr' Dalem
 * 
 * This file is part of Demyo. Demyo is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License version 3,
 * as published by the Free Software Foundation.
 * 
 * Demyo is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU GPLv3 for more details.
 */
class MenuHelper extends AppHelper {
	public $helpers = array('jQuery', 'Html');
	
	private $titles;
	
	public function beforeRender() {
		$this->titles = array(
			'add' => array(
				'Album' => __('Add an album', true),
				'Author' => __('Add an author', true),
				'Binding' => __('Add a binding', true),
				'Borrower' => __('Add a borrower', true),
				'Collection' => __('Add a collection', true),
				'Derivative' => __('Add a derivative', true),
				'DerivativeType' => __('Add a derivative type', true),
				'Image' => __('Add an image', true),
				'Publisher' => __('Add a publisher', true),
				'Series' => __('Add a series', true),
				'Source' => __('Add a source', true),
				'Tag' => __('Add a tag', true)
			),
			'edit' => array(
				'Album' => __('Edit this album', true),
				'Author' => __('Edit this author', true),
				'Binding' => __('Edit this binding', true),
				'Borrower' => __('Edit this borrower', true),
				'Collection' => __('Edit this collection', true),
				'Derivative' => __('Edit this derivative', true),
				'DerivativeType' => __('Edit this derivative type', true),
				'Image' => __('Edit this image', true),
				'Publisher' => __('Edit this publisher', true),
				'Series' => __('Edit this series', true),
				'Source' => __('Edit this source', true),
				'Tag' => __('Rename this tag', true)
			),
			'delete' => array(
				'Album' => __('Delete this album', true),
				'Author' => __('Delete this author', true),
				'Binding' => __('Delete this binding', true),
				'Borrower' => __('Delete this borrower', true),
				'Collection' => __('Delete this collection', true),
				'Derivative' => __('Delete this derivative', true),
				'DerivativeType' => __('Delete this derivative type', true),
				'Image' => __('Delete this image', true),
				'Publisher' => __('Delete this publisher', true),
				'Series' => __('Delete this series', true),
				'Source' => __('Delete this source', true),
				'Tag' => __('Delete this tag', true)
			),
			'delete-confirm' => array(
				'Album' => __('Are you sure you want to delete this album?', true),
				'Author' => __('Are you sure you want to delete this author?', true),
				'Binding' => __('Are you sure you want to delete this binding?', true),
				'Borrower' => __('Are you sure you want to delete this borrower?', true),
				'Collection' => __('Are you sure you want to delete this collection?', true),
				'Derivative' => __('Are you sure you want to delete this derivative?', true),
				'DerivativeType' => __('Are you sure you want to delete this derivative type?', true),
				'Image' => __('Are you sure you want to delete this image?', true),
				'Publisher' => __('Are you sure you want to delete this publisher?', true),
				'Series' => __('Are you sure you want to delete this series?', true),
				'Source' => __('Are you sure you want to delete this source?', true),
				'Tag' => __('Are you sure you want to delete this tag?', true)
			),
		);
	}
	
	/** Outputs or echoes a popup menu.
	 * 
	 * The $items array is an array of items, each being defined as follows:
	 *  - An array of the type ('action' => 'Model'), where action can be 'add',
	 *    'edit' or 'delete'. For 'edit' and 'delete', a ('id' => $id) value
	 *    should be added to the array.
	 *  - Values for the keys 'url' (not passed through Router::url()), 'title',
	 *    'escape' and 'confirm'. The first two properties override the default
	 *    URLs/strings from 'add'/'edit'/'delete' if specified alognside them.
	 *    Oterwise, they're enough to form a valid item (without specifying an
	 *    action or model).
	 * 
	 * @param array $items The items array. See doc above.
	 * @param string $menuTitle The title of the menu. Defaults to localized 'Menu' string.
	 * @param string $menuID The (X)HTML id of the menu
	 * @param bool $echo True if the result should be echoed directly
	 * @return string The result string (if $echo is set to false)
	 */
	public function makePopupMenu($items = array(), $menuTitle = false, $menuID = 'quickmenu', $echo = true) {
		$this->jQuery->checkInclude();
		
		if ($menuTitle === false)
			$menuTitle = __('Tasks', true);
			
		$menuString = '<div id="'.$menuID.'">'
			."\n\t<h1>$menuTitle</h1>"
			."\n\t<ul>";
		
		foreach ($items as $item) {
			$confirm = false;
			$escape = true;
			
			// Parse item
			if (!empty($item['add'])) {
				$url = array('controller' => lcfirst(Inflector::pluralize($item['add'])), 'action' => 'add');
				$title = $this->titles['add'][$item['add']];
			}
			
			if (!empty($item['edit'])) {
				$url = array('controller' => lcfirst(Inflector::pluralize($item['edit'])), 'action' => 'edit', $item['id']);
				$title = $this->titles['edit'][$item['edit']];
			}
			
			if (!empty($item['delete'])) {
				$url = array('controller' => lcfirst(Inflector::pluralize($item['delete'])), 'action' => 'delete', $item['id']);
				$title = $this->titles['delete'][$item['delete']];
				$confirm = $this->titles['delete-confirm'][$item['delete']];
			}
			
			extract($item);
			
			// Build markup
			$menuString .= "\n\t\t<li>"
				.$this->Html->link($title, $url, array(), $confirm, $escape)
				.'</li>';
		}
		
		$menuString .= "\n\t</ul>\n</div>";
		
		$menuString .= <<<EOS
<script type="text/javascript">
var \$jq = jQuery.noConflict();
\$jq('div#$menuID').hover(
	function() {
		// Prevent flickering when multiple hovers are chained (for example
		// because you missed the button)
		\$jq('div#$menuID > ul').stop();
		\$jq('div#$menuID > ul').css('opacity', 1);
		\$jq('div#$menuID > ul').fadeIn('fast');
		\$jq('div#$menuID h1').css('border-bottom-color', \$jq('div#$menuID h1').css('background-color'));
	},
	function() {
		// Idem
		\$jq('div#$menuID > ul').stop();
		\$jq('div#$menuID > ul').css('opacity', 1);
		\$jq('div#$menuID > ul').fadeOut('fast');
		\$jq('div#$menuID h1').css('border-bottom-color', \$jq('div#$menuID h1').css('border-left-color'));
	}
	);
</script>

EOS;
		
		if ($echo === true)
			echo $menuString;
		else
			return $menuString;
	}
}
?>