<?php
/* Copyright 2007-2009 Xavier 'Xr' Dalem
 * 
 * This file is part of Demyo. Demyo is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License version 3,
 * as published by the Free Software Foundation.
 * 
 * Demyo is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU GPLv3 for more details.
 */

App::import('Helper', 'Paginator');

class MyPaginatorHelper extends PaginatorHelper {
	/**
	 * Generates a "previous" link for a set of paged records
	 *
	 * @param  string $title Title for the link. Defaults to '<< Previous'.
	 * @param  mixed $options Options for pagination link. See #options for list of keys.
	 * @param  string $disabledTitle Title when the link is disabled.
	 * @param  mixed $disabledOptions Options for the disabled pagination link. See #options for list of keys.
	 * @return string A "previous" link or $disabledTitle text if the link is disabled.
	 */
	function prev($title = '«', $options = array(), $disabledTitle = null, $disabledOptions = array()) {
		return parent::prev($title, $options, $disabledTitle, $disabledOptions);
	}
	
	/**
	 * Generates a "next" link for a set of paged records
	 *
	 * @param  string $title Title for the link. Defaults to 'Next >>'.
	 * @param  mixed $options Options for pagination link. See #options for list of keys.
	 * @param  string $disabledTitle Title when the link is disabled.
	 * @param  mixed $disabledOptions Options for the disabled pagination link. See #options for list of keys.
	 * @return string A "next" link or or $disabledTitle text if the link is disabled.
	 */
	function next($title = '»', $options = array(), $disabledTitle = null, $disabledOptions = array()) {
		return parent::next($title, $options, $disabledTitle, $disabledOptions);
	}
	
	/**
	 * Returns a set of numbers for the paged result set
	 * uses a modulus to decide how many numbers to show on each side of the current page (default: 8)
	 *
	 * @param  mixed $options Options for the numbers, (before, after, model, modulus, separator)
	 * @return string numbers string.
	 */
	function numbers($options = array()) {
		if ($options === true) {
			$options = array(
						'before' => ' | ', 'after' => ' | ',
						'first' => 'first', 'last' => 'last',
						);
		}

		$options = array_merge(
			array(
				'tag' => 'span',
				'before'=> null, 'after'=> null,
				'model' => $this->defaultModel(),
				'modulus' => 6, 'quickJump' => 8,
				'separator' => ', ', 'jumpSeparator' => ' ... ',
				'first' => 2, 'last' => 2,
			),
		(array)$options);

		$params = array_merge(array('page'=> 1), (array)$this->params($options['model']));
		unset($options['model']);

		if ($params['pageCount'] <= 1) {
			return false;
		}
		
		extract($options);
		unset($options['tag'], $options['before'], $options['after'], $options['model'],
			$options['modulus'], $options['separator'], $options['first'], $options['last'],
			$options['quickJump'], $options['jumpSeparator']);

		$out = '';
		
		if ($quickJump <= floor($modulus/2)) { // It would be useless
			$quickJump = null;
		}

		if ($modulus && $params['pageCount'] > $modulus) {
			$half = floor($modulus / 2);
			
			$end = $params['page'] + $half;
			if ($end > $params['pageCount']) {
				$end = $params['pageCount'];
			}
			
			$start = $params['page'] - $half;
			if ($start <= 1) {
				$start = 1;
				$end = $modulus + 1;
			}

			// If the first numbers to be printed and the start of the count
			// overlap, merge them. This way, more numbers will be printed
			// during the first pages but the links to the first pages will
			// never go missing
			if (intval($first) && $start <= $first)
				$start = 1;
			
			// Links to first pages
			if ($first && $start > (int)$first) {
				if ($start == $first + 1) {
					$out .= $this->first($first, array('tag' => $tag, 'separator' => $separator, 'after' => $separator));
				} else {
					$out .= $this->first($first, array('tag' => $tag, 'separator' => $separator, 'after' => $jumpSeparator));
				}
			}

			$out .= $before;
			
			// Quick jump backwards
			if ($quickJump !== null) {
				$quickJumpPos = $params['page'] - $quickJump;
				if ($quickJumpPos > (is_int($first)?$first+1:0) && $quickJumpPos < $start-1)
					$out .= ($first?'':$jumpSeparator) . $this->Html->tag($tag, $this->link($quickJumpPos, array('page' => $quickJumpPos), $options)) . $jumpSeparator;
			}

			// Links to pages before the current one
			for ($i = $start; $i < $params['page']; $i++) {
				$out .= $this->Html->tag($tag, $this->link($i, array('page' => $i), $options)) . $separator;
			}

			$out .= $this->Html->tag($tag, $params['page'], array('class' => 'current'));
			if ($i != $params['pageCount']) {
				$out .= $separator;
			}
			
			// Symetric of the start link tweak above
			if (intval($last) && $end >= $params['pageCount'] - (int)$last)
				$end = $params['pageCount'];

			// Links to pages after the current one
			$start = $params['page'] + 1;
			for ($i = $start; $i < $end; $i++) {
				$out .= $this->Html->tag($tag, $this->link($i, array('page' => $i), $options)). $separator;
			}

			if ($end != $params['page']) {
				$out .= $this->Html->tag($tag, $this->link($i, array('page' => $end), $options));
			}
			
			// Quick jump forward
			if ($quickJump !== null) {
				$quickJumpPos = $params['page'] + $quickJump;
				if ($quickJumpPos < $params['pageCount'] - (is_int($last)?$last:0) && $quickJumpPos > $end+1)
					$out .= $jumpSeparator . $this->Html->tag($tag, $this->link($quickJumpPos, array('page' => $quickJumpPos), $options)) . ($last?'':$jumpSeparator);
			}

			$out .= $after;
			
			// Links to last pages
			if ($last && $end <= $params['pageCount'] - (int)$last) {
				if ($end + 1 == $params['pageCount']) {
					$out .= $this->last($last, array('tag' => $tag, 'before' => $separator));
				} else {
					$out .= $this->last($first, array('tag' => $tag, 'separator' => $separator, 'before' => $jumpSeparator));
				}
			}

		} else {
			$out .= $before;

			for ($i = 1; $i <= $params['pageCount']; $i++) {
				if ($i == $params['page']) {
					$out .= $this->Html->tag($tag, $i, array('class' => 'current'));
				} else {
					$out .= $this->Html->tag($tag, $this->link($i, array('page' => $i), $options));
				}
				if ($i != $params['pageCount']) {
					$out .= $separator;
				}
			}

			$out .= $after;
		}

		return $this->output($out);
	}
	
	public function directJump($options = array(), $title = false) {
		if ($title === false)
			$title = __('Jump to page', true);
		
		$options = array_merge(array('model' => $this->defaultModel()),
			$this->options, $options);
		$params = array_merge(array('page'=> 1), (array)$this->params($options['model']));

		if ($params['pageCount'] <= 1) {
			return false;
		}
		
		if (isset($options['url']))
			$url = $options['url'];
		else
			$url = array();

		$thisPage = $params['page'];
		$output = "$title&nbsp;<select onchange=\"location.href = this.value\">\n";
		for ($i = 1; $i <= $params['pageCount']; $i++) {
			if ($i == $thisPage)
				$output .= "\t<option selected=\"selected\" value=\"".$this->url(array_merge($url, array('page' => $i)))."\">$i</option>\n";
			else
				$output .= "\t<option value=\"".$this->url(array_merge($url, array('page' => $i)))."\">$i</option>\n";
		}
		$output .= "</select>\n";
		
		return $output;
	}
	
	public function letters($options = array()) {
		if (empty($this->params['paging']))
			return false;
		
		$options = array_merge(array(
				'model' => $this->defaultModel(),
				'url' => array()
			), $this->options, $options);
		$url = array_merge($this->params['pass'], $options['url']);
		
		$params = array_merge(array('page'=> 1), (array)$this->params($options['model']));

		if ($params['pageCount'] <= 1) {
			return false;
		}
		
		$output = '';
		$thisPage = $this->params['paging'][$options['model']]['options']['page'];
		unset($this->params['paging'][$options['model']]['options']['page']);
		
		foreach (range('a', 'z') as $l)
			$output .= '<a title="'.__s('Jump to items starting with \'%s\'', $l).'" href="'.$this->url(array_merge($url, array('startsWith' => $l)))."\">$l</a>\n";
			
		$this->params['paging'][$options['model']]['options']['page'] = $thisPage;
		
		return $output;
	}
}

?>